/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 * Base abstract interface and audio symbols definition.
 *
 * Audio drivers have to be derived from class Audio and follow
 * these rules:
 *	- put their configuration in the argument of openDevice
 *	- signal a recoverable error returning false
 *	- signal an unrecoverable error throwing an exception AudioError
 *
 * If control over buffering is supported the driver is expected to:
 *	- set as default if fragSize == -1
 *	- if only fragNumber == -1 set only the frag. size
 */

#ifndef _AUDIO_H_
#define _AUDIO_H_

#include "conffile.h"
#include "audioerror.h"

namespace Tuner {
	enum OpenMode {
		PLAYBACK,
		RECORD,
		DUPLEX
	};

	enum Format {
		U8,
		S16BE,
		S16LE
	};

	enum Channels {
		MONO,
		STEREO
	};

	typedef	int SamplingRate;

	struct AudioInfo {
		OpenMode	mode;
		int		fragNumber;
		int		fragSize;
		Format		fmt;
		Channels	chn;
		SamplingRate 	rate;

		bool	load(Config::ConfFile &) throw(Error::AudioError);
		bool	loadDefaults() throw();
		bool	save(Config::ConfFile &) throw(Error::AudioError);

		int		sampleSize() const throw();
	};

	inline int
	AudioInfo::sampleSize() const throw()
	{ return (fmt == U8 ? 1: 2) * (chn + 1); }

	class Audio {
		public:
		virtual	    ~Audio();

		virtual	bool	openDevice(AudioInfo *, bool, bool)
			throw(Error::AudioError) = 0;
		virtual	bool	closeDevice() throw() = 0;

		virtual	bool	play(unsigned char *, int)
			throw(Error::AudioError) = 0;
		virtual	bool	record(unsigned char *, int)
			throw(Error::AudioError) = 0;

		virtual	bool	avail(int *, int *)
			throw(Error::AudioError) = 0;
		virtual	bool	free(int *, int *)
			throw(Error::AudioError) = 0;

		virtual	bool	isOpen() throw() = 0;

		virtual	bool	sync() throw(Error::AudioError) = 0;
		virtual	bool	stop() throw(Error::AudioError) = 0;

		virtual const AudioInfo *getInfo() throw() = 0;
	};

	inline Audio::~Audio() { }
}

#endif

