/*  GuiTuner 0.05
 *  Copyright (C) 2000 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 *  This isn' t really a template, but an easy way to write to classes
 *  (i.e. Fft<unsigned char> and Fft<int>):
 *  there can' t be declared Fft<double> or Fft<float> as it should be,
 *  because this program does not need them.
 */

#ifndef _FFT_H_
#define _FFT_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "error.h"
#include "conf.h"
#include "gate.h"
#include "window.h"

#ifdef HAVE_FFTW
#include <rfftw.h>
#else

#include <cmath>
#include <cstdlib>

#ifdef M_PI
#define PI	M_PI
#else
#define PI	3.14159265358979323846
#endif

#endif

namespace Tuner {
	template <typename T>
	class Fft {
		public:
			Fft(int, ByteOrder, Window::WindowType)
				throw(std::bad_alloc);
		       ~Fft();

		void	reconfigure(int, ByteOrder, Window::WindowType)
			throw(std::bad_alloc);

		bool	calc(T *, double *) throw();
		bool	_calc(unsigned char *, double *) throw();

		int		getPoints() const throw();
		int	outputSize() throw();

		private:
			Fft(const Fft &);
		const Fft &operator=(const Fft &);

		void	applyWindow(T *) throw();
		void	_applyWindow(unsigned char *) throw();

		void	init(int) throw(std::bad_alloc);
		void	destroy() throw();

		Window win;
		int points;
		ByteOrder order;
#ifdef HAVE_FFTW
		rfftw_plan plan;
		fftw_real *in, *out;
#else
		int	reverseBits(int) throw();
		int	log2(int) throw();
		void	doTransform() throw();

		double *cosTable, *sinTable;
		double *realOut, *imagOut;
		int *indexTable, lognn;
#endif
	};


	template <typename T>
	inline Fft<T>::Fft(int n, ByteOrder o, Window::WindowType t)
		throw(std::bad_alloc): win(n, t), order(o)
	{ init(n); }

	template <typename T>
	inline Fft<T>::~Fft()
	{ destroy(); }

	template <typename T>
	inline void Fft<T>::reconfigure(int n, ByteOrder o,
		Window::WindowType t) throw(std::bad_alloc)
	{ win.reconfigure(n, t), order = o, destroy(), init(n); }

	template <typename T>
	inline int Fft<T>::getPoints() const throw()
	{ return points; }

	template <typename T>
	inline int Fft<T>::outputSize() throw()
	{ return points / 2 + 1; }
}

#endif

