/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _MIDINOTE_H_
#define _MIDINOTE_H_

#include <string>
#include <cstdlib>

namespace Player {
	class MidiNote {
		public:
			MidiNote(int = 0) throw();
			MidiNote(int, int) throw();
		//	MidiNote(const MidiNote &);
		//	~MidiNote();

		//const MidiNote &operator=(const MidiNote &);
		const std::string readName() const throw();
		double frequency() const throw();

		private:
		int midiVal;
	};

	inline MidiNote::MidiNote(int n) throw(): midiVal(n) { }
	inline MidiNote::MidiNote(int n, int o) throw(): midiVal(o*12+n) { }

	inline const std::string MidiNote::readName() const throw()
	{
		const char *Name[] = { "C", "C#", "D", "D#", "E", "F", "F#", "G",
			"G#", "A", "A#", "B" };
		return std::string(Name[midiVal%12]);
	}
	inline double MidiNote::frequency() const throw()
	{
		const unsigned int freqs[] = { 261632, 277189, 293671, 311132,
			329632, 349232, 369998, 391998, 415306, 440000, 466162, 493880 };
		const int BaseOctave = 4;

		int o = midiVal / 12, n = midiVal % 12;
		unsigned int freq = freqs[n];
		if(o < BaseOctave)
			freq >>= (BaseOctave - o);
		else if(BaseOctave < o)
			freq <<= (o - BaseOctave);

		return static_cast<double>(freq) / 1000;
	}
}

#endif

