/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <unistd.h>
#include "playerthread.h"
#include "presetdata.h"
#include "utils.h"
#include "i18n.h"

Player::PlayerThread::PlayerThread(Config::GenConf &c)throw(Error::AudioError, 
	Error::ThreadError, std::bad_alloc):  fact(c), selected(0),  
	audioDev(fact.createAudioPlay()), devInfo(c.getAudioInfoPlay()), buf(0), playing(false)
{
	loadPresets();
}

void
Player::PlayerThread::play(int string) throw(Error::PlayerError, Error::ThreadError)
{
	if(!playing) {
		Command cmd = PLAY;
		sendMsg(&cmd, sizeof(cmd));
		sendMsg(&string, sizeof(int));
	}
}

void
Player::PlayerThread::start() throw(Error::AudioError, Error::PlayerError,
	Error::ThreadError)
{
	if(!audioDev->openDevice(&devInfo, false, false))
		throw Error::PlayerError(std::string(_("PlayerThread::start()")),
			std::string(_("Cannot open audio device")));

	PosixThread::run();
}

void
Player::PlayerThread::stop() throw(Error::ThreadError)
{
	Command cmd = STOP;
	sendMsg(&cmd, sizeof(cmd));
	PosixThread::wait();
}

void
Player::PlayerThread::loadPresets() throw()
{
	presets.resize(DEF_PRESETS);

	for(int i = 0; i < DEF_PRESETS; ++i) {
		vector<MidiNote> v(6);
		for(int k = 0; k < 6; ++k)
			v[k] = MidiNote(stdPreset[i].val[k]);
		presets[i] = Preset(stdPreset[i].name, v);
	}
}

int
Player::PlayerThread::main() throw()
{
	using namespace Thread;
	using namespace Tuner;

	try {
		while(1) {
			if(pending(FROM_PARENT)) {
				Command cmd;
				getMsg(&cmd, sizeof(cmd), FROM_PARENT);
				switch(cmd) {
					case PLAY: {
						int n;
						getMsg(&n, sizeof(int), FROM_PARENT);
						if(tone(buf, presets[selected][n].frequency(), devInfo))
							playing = true;
					}
					break;
					case STOP:
						audioDev->closeDevice();
						return 0;
				}
			}
			if(playing) {
				audioDev->play(buf.g_ptr(), buf.g_size());
				playing = false;
			} else {
				usleep(100000);
			}
		}
	} catch(std::exception &) {
		return -1;
	}
}

